### Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
""" wp-cli utility wrapper to execute it under specified system user. """
import os
import sys
import subprocess
from optparse import OptionParser
from plesk_exec import drop_privileges, chdir_ignore_failure

CLIENT_PHP_BIN = "/opt/psa/bin/php-cli"

import plesk_log; plesk_log.setDefaultLevel('warning')
log = plesk_log.getLogger(__name__)

def doMain(argv):
    parser = OptionParser(usage="Usage: %prog --user <username> [--php <php_interpreter>] -- <wp-cli args>")
    parser.add_option("-u", "--user", help="system user")
    parser.add_option("-p", "--php", help="path to php interpreter")
    (options, args) = parser.parse_args()

    drop_privileges(options.user, "psacln")
    chdir_ignore_failure('/tmp', log) or chdir_ignore_failure('/', log)

    os.environ['WP_CLI_PHP'] = options.php if options.php else CLIENT_PHP_BIN
    os.environ['WP_CLI_PHP_ARGS'] = "-d safe_mode=off -d display_errors=off"
    os.environ['WP_CLI_CONFIG_PATH'] = "/usr/share/plesk-wp-cli/wp-cli.yml"
    sys.exit(subprocess.call(["/usr/share/plesk-wp-cli/bin/wp"] + args))

def main():
    try:
        doMain(sys.argv)
    except SystemExit:
        raise
    except Exception as ex:
        log.error('%s', ex)
        log.debug('This exception happened at:', exc_info=sys.exc_info())
        sys.exit(1)

if __name__ == '__main__':
	main()

# vim: ts=4 sts=4 sw=4 et :
